<?php

/**
 * @file The administration interface.
 */

/**
 * List all transactions.
 *
 * @return string
 */
function idealui_transaction_list() {
  global $pager_page_array, $pager_total, $pager_total_items;

  // Build the pager. We use Drupal's built-in paging system, but don't use any
  // of the API functions, because they are too SQL-dependent.
  $page = isset($_GET['page']) ? $_GET['page'] : '';
  $pager_page_array = explode(',', $page);
  $pager_total_items[0] = db_result(db_query("SELECT COUNT(1) FROM {ideal_transaction}"));
  $pager_total[0] = ceil($pager_total_items[0] / 20);
  $pager = theme('pager');

  // Build the table.
  $header = array(
    array(
      'data' => t('Last update'),
      'field' => 'updated',
      'sort' => 'desc',
    ),
    array(
      'data' => t('Amount'),
      'field' => 'amount',
    ),
    array(
      'data' => t('Description'),
      'field' => 'description',
    ),
    array(
      'data' => t('Name'),
      'field' => 'name',
    ),
    array(
      'data' => t('City'),
      'field' => 'city',
    ),
    array(
      'data' => t('Account number'),
      'field' => 'account_number',
    ),
    t('Status'),
  );
  $sort_field = 'updated';
  $order = 'desc';
  if (isset($_GET['order'])) {
    foreach ($header as $cell) {
      if ($cell['data'] == $_GET['order']) {
        $sort_field = $cell['field'];
        $order = isset($_GET['sort']) ? $_GET['sort'] : $cell['sort'];
      }
    }
  }
  $transactions = ideal_transaction_load_multiple(array(), $sort_field, $order, $page * 20, 20);
  $rows = array();
  foreach ($transactions as $transaction) {
    $unknown = $transaction->status == IDEAL_STATUS_UNKNOWN;
    $rows[] = array(
      'data' => array(
        format_date($transaction->updated, 'small'),
        '€' . ideal_cents_to_human_readable($transaction->amount),
        $transaction->description,
        $transaction->name,
        $transaction->city,
        $transaction->account_number,
        $unknown ? theme('idealui_status_unknown', $transaction) : ideal_status_title($transaction->status),
      ),
      'class' => $unknown ? 'error' : NULL,
    );
  }
  if (!$rows) {
    $rows[] = array(
      array(
        'data' => t('There are no transactions yet.'),
        'colspan' => 7,
      ),
    );
  }

  return $pager . theme('table', $header, $rows) . $pager;
}

/**
 * Display IDEAL_STATUS_UNKNOWN in the transaction overview.
 */
function theme_idealui_status_unknown(iDEALTransaction $transaction) {
  return ideal_status_title(IDEAL_STATUS_UNKNOWN) . '<div class ="description">' . t("Contact your bank to check this transaction's status.") . '</div>';
}

/**
 * The merchant overview form.
 */
function idealui_form_merchant_list(array &$form_state) {
  foreach (ideal_merchant_load_all() as $merchant) {
    if ($merchant->imid == variable_get('ideal_default_merchant', 1)) {
      $form[$merchant->imid]['default'] = array(
        '#value' => t('Default merchant'),
      );
    }
    else {
      $form[$merchant->imid]['default'] = array(
        '#type' => 'submit',
        '#value' => t('Set as default'),
        '#name' => 'merchant-' . $merchant->imid,
      );
    }
    if (idealui_access_merchant($merchant, 'edit')) {
      $form[$merchant->imid]['edit'] = array(
        '#value' => l(t('edit'), 'admin/settings/ideal/merchant/edit/' . $merchant->imid),
      );
    }
    if (idealui_access_merchant($merchant, 'delete')) {
      $form[$merchant->imid]['delete'] = array(
        '#value' => l(t('delete'), 'admin/settings/ideal/merchant/delete/' . $merchant->imid),
      );
    }
  }

  return $form;
}

/**
 * Theme callback for idealui_form_merchant_list().
 */
function theme_idealui_form_merchant_list(array $form) {
  $description = '<div class="description">' . t("The default merchant will be used by modules that aren't configured to use a specific merchant.") . '</div>';

  $rows = array();
  foreach (ideal_merchant_load_all() as $merchant) {
    $rows[] = array(
      $merchant->title,
      $merchant->merchant_id,
      $merchant->sub_id,
      $merchant->acquirer->title,
      drupal_render($form[$merchant->imid]['default']),
      idealui_access_merchant($merchant, 'edit') ? drupal_render($form[$merchant->imid]['edit']) : NULL,
      idealui_access_merchant($merchant, 'delete') ? drupal_render($form[$merchant->imid]['delete']) : NULL,
    );
  }
  $header = array(
    t('Title'),
    t('Merchant ID'),
    t('Sub ID'),
    t('Acquirer'),
    array(
      'data' => t('Operations'),
      'colspan' => 3,
    ),
  );

  return drupal_render($form) . theme('table', $header, $rows) . $description;
}

/**
 * Submit callback for idealui_form_merchant_list().
 */
function idealui_form_merchant_list_submit(array $form, array &$form_state) {
  $imid = str_replace('merchant-', '', $form_state['clicked_button']['#name']);
  variable_set('ideal_default_merchant', $imid);
  $merchant = ideal_merchant_load($imid);
  drupal_set_message(t('%merchant has been set as the default merchant.', array(
    '%merchant' => $merchant->title,
  )));
}

/**
 * The merchant add/edit form.
 */
function idealui_form_merchant(array &$form_state, iDEALMerchant $merchant = NULL) {
  $form['imid'] = array(
    '#type' => 'value',
    '#value' => $merchant ? $merchant->imid : 0,
  );
  $form['title'] = array(
    '#type' => 'textfield',
    '#title' => t('Title'),
    '#default_value' => $merchant ? $merchant->title : NULL,
    '#maxlength' => 255,
    '#required' => TRUE,
  );
  $options = array('<' . t('none') . '>');
  foreach (ideal_acquirer_load_all() as $acquirer) {
    $options[$acquirer->iaid] = $acquirer->title;
  }
  $form['iaid'] = array(
    '#type' => 'select',
    '#title' => t('Acquirer'),
    '#default_value' => $merchant ? $merchant->acquirer->iaid : 0,
    '#options' => $options,
    '#required' => TRUE,
  );
  $form['merchant_id'] = array(
    '#type' => 'textfield',
    '#title' => t('Merchant ID'),
    '#default_value' => $merchant ? $merchant->merchant_id : NULL,
    '#maxlength' => 9,
    '#size' => 9,
    '#required' => TRUE,
  );
  $form['sub_id'] = array(
    '#type' => 'textfield',
    '#title' => t('Sub ID'),
    '#default_value' => $merchant ? $merchant->sub_id : NULL,
    '#maxlength' => 6,
    '#size' => 6,
  );
  $form['private_key_path'] = array(
    '#type' => 'textfield',
    '#title' => t('Path to private key file'),
    '#default_value' => $merchant ? $merchant->private_key_path : NULL,
    '#maxlength' => 255,
    '#required' => TRUE,
  );
  $form['private_key_password'] = array(
    '#type' => 'textfield',
    '#title' => t('Private key password'),
    '#default_value' => $merchant ? $merchant->private_key_password : NULL,
    '#description' => t('This password will be stored as plain text.'),
    '#maxlength' => 255,
  );
  $form['private_certificate_path'] = array(
    '#type' => 'textfield',
    '#title' => t('Path to private certificate file'),
    '#default_value' => $merchant ? $merchant->private_certificate_path : NULL,
    '#maxlength' => 255,
    '#required' => TRUE,
  );
  $form['buttons']['save'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );
  if ($merchant && idealui_access_merchant($merchant, 'delete')) {
    $form['buttons']['delete'] = array(
      '#value' => l(t('Delete'), 'admin/settings/ideal/merchant/delete/' . $merchant->imid),
    );
  }

  return $form;
}

/**
 * Validate callback for idealui_form_merchant().
 */
function idealui_form_merchant_validate(array $form, array &$form_state) {
  $values = $form_state['values'];
  if (preg_match('#[^\da-z]#i', $values['merchant_id'])) {
    form_set_error('merchant_id', t('The merchant ID must be alphanumeric.'));
  }
  if (preg_match('#[\D]#i', $values['sub_id'])) {
    form_set_error('sub_id', t('The sub ID must be numeric.'));
  }
  if ($values['iaid'] == 0) {
    form_set_error('acquirer', t('Acquirer field is required.'));
  }
  if (!is_readable($values['private_key_path'])) {
    form_set_error('private_key_path', t('The private key file is not readable.'));
  }
  elseif (!@openssl_pkey_get_private('file://' . $values['private_key_path'], $values['private_key_password'])) {
    form_set_error('private_key_path', t('The private key could not be extracted from the private key file.'));
  }
  if (!is_readable($values['private_certificate_path'])) {
    form_set_error('private_certificate_path', t('The private certificate file is not readable.'));
  }
  else {
    $certificate = NULL;
    if (!($x509 = @openssl_x509_read(file_get_contents($values['private_certificate_path']))) || !@openssl_x509_export($x509, $certificate)) {
      form_set_error('private_certificate_path', t('The private certificate could not be extracted from the private certificate file.'));
    }
  }
}

/**
 * Submit callback for idealui_form_merchant().
 */
function idealui_form_merchant_submit(array $form, array &$form_state) {
  $values = $form_state['values'];
  $merchant = new iDEALMerchant(array(
    'imid' => $values['imid'],
    'title' => $values['title'],
    'merchant_id' => $values['merchant_id'],
    'sub_id' => $values['sub_id'],
    'acquirer' => ideal_acquirer_load($values['iaid']),
    'private_key_path' => $values['private_key_path'],
    'private_key_password' => $values['private_key_password'],
    'private_certificate_path' => $values['private_certificate_path'],
  ));
  ideal_merchant_save($merchant);
  $form_state['redirect'] = 'admin/settings/ideal/merchant';
  drupal_set_message(t('The merchant %title has been saved.', array(
    '%title' => $merchant->title,
  )));
}

/**
 * Merchant deletion form.
 */
function idealui_form_merchant_delete(array &$form_state, iDEALMerchant $merchant) {
  $form['imid'] = array(
    '#type' => 'value',
    '#value' => $merchant->imid,
  );
  return confirm_form($form, t('Do you really want to delete merchant %title?', array(
    '%title' => $merchant->title,
  )), 'admin/settings/ideal/merchant');
}

/**
 * Submit callback for idealui_form_merchant_delete().
 */
function idealui_form_merchant_delete_submit(array $form, array &$form_state) {
  $imid = $form_state['values']['imid'];
  $merchant = ideal_merchant_load($imid);
  ideal_merchant_delete($imid);
  $form_state['redirect'] = 'admin/settings/ideal/merchant';
  drupal_set_message(t('The merchant %title has been deleted.', array(
    '%title' => $merchant->title,
  )));
}

/**
 * The acquirer overview.
 */
function idealui_acquirer_list() {
  $rows = array();
  foreach (ideal_acquirer_load_all() as $acquirer) {
    $rows[] = array(
      $acquirer->title,
      idealui_access_acquirer($acquirer, 'edit') ? l(t('edit'), 'admin/settings/ideal/acquirer/edit/' . $acquirer->iaid) : NULL,
      idealui_access_acquirer($acquirer, 'delete') ? l(t('delete'), 'admin/settings/ideal/acquirer/delete/' . $acquirer->iaid) : NULL,
    );
  }
  $header = array(
    t('Title'),
    array(
      'data' => t('Operations'),
      'colspan' => 2,
    ),
  );

  return theme('table', $header, $rows);
}

/**
 * The acquirer add/edit form.
 */
function idealui_form_acquirer(array &$form_state, iDEALAcquirer $acquirer = NULL) {
  $form['iaid'] = array(
    '#type' => 'value',
    '#value' => $acquirer ? $acquirer->iaid : 0,
  );
  $form['title'] = array(
    '#type' => 'textfield',
    '#title' => t('Title'),
    '#default_value' => $acquirer ? $acquirer->title : NULL,
    '#maxlength' => 255,
    '#required' => TRUE,
  );
  $form['url_live'] = array(
    '#type' => 'textfield',
    '#title' => t('Production server URL'),
    '#default_value' => $acquirer ? $acquirer->url_live : NULL,
    '#maxlength' => 255,
    '#required' => TRUE,
  );
  $form['url_test'] = array(
    '#type' => 'textfield',
    '#title' => t('Testing server URL'),
    '#default_value' => $acquirer ? $acquirer->url_test : NULL,
    '#maxlength' => 255,
    '#required' => TRUE,
  );
  $form['public_certificate_path'] = array(
    '#type' => 'textfield',
    '#title' => t('Path to public certificate file'),
    '#default_value' => $acquirer ? $acquirer->public_certificate_path : NULL,
    '#maxlength' => 255,
    '#required' => TRUE,
  );
  $form['buttons']['save'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );
  if ($acquirer && idealui_access_acquirer($acquirer, 'delete')) {
    $form['buttons']['delete'] = array(
      '#value' => l(t('Delete'), 'admin/settings/ideal/acquirer/delete/' . $acquirer->iaid),
    );
  }

  return $form;
}

/**
 * Validate callback for idealui_form_acquirer().
 */
function idealui_form_acquirer_validate(array $form, array &$form_state) {
  $values = $form_state['values'];
  if (!valid_url($values['url_live'], TRUE)) {
    form_set_error('url_live', t('The production server URL is invalid.'));
  }
  if (!valid_url($values['url_test'], TRUE)) {
    form_set_error('url_live', t('The testing server URL is invalid.'));
  }
  if (!is_readable($values['public_certificate_path'])) {
    form_set_error('public_certificate_path', t('The public certificate file is not readable.'));
  }
  elseif (!@openssl_pkey_get_public('file://' . $values['public_certificate_path'])) {
    form_set_error('public_certificate_path', t('The public certificate could not be extracted from the public certificate file.'));
  }
}

/**
 * Submit callback for idealui_form_acquirer().
 */
function idealui_form_acquirer_submit(array $form, array &$form_state) {
  module_load_include('inc', 'ideal', '/includes/ideal.transaction');
  $values = $form_state['values'];
  $acquirer = new iDEALAcquirer(array(
    'iaid' => $values['iaid'],
    'title' => $values['title'],
    'url_live' => $values['url_live'],
    'url_test' => $values['url_test'],
    'public_certificate_path' => $values['public_certificate_path'],
  ));
  ideal_acquirer_save($acquirer);
  $form_state['redirect'] = 'admin/settings/ideal/acquirer';
  drupal_set_message(t('The acquirer %title has been saved.', array(
    '%title' => $acquirer->title,
  )));
}

/**
 * Acquirer deletion form.
 */
function idealui_form_acquirer_delete(array &$form_state, iDEALAcquirer $acquirer) {
  $form['imid'] = array(
    '#type' => 'value',
    '#value' => $acquirer->iaid,
  );
  return confirm_form($form, t('Do you really want to delete acquirer %title?', array(
    '%title' => $acquirer->title,
  )), 'admin/settings/ideal/acquirer');
}

/**
 * Submit callback for idealui_form_acquirer_delete().
 */
function idealui_form_acquirer_delete_submit(array $form, array &$form_state) {
  $iaid = $form_state['values']['iaid'];
  $merchant = ideal_acquirer_load($iaid);
  ideal_acquirer_delete($imid);
  $form_state['redirect'] = 'admin/settings/ideal/acquirer';
  drupal_set_message(t('The acquirer %title has been deleted.', array(
    '%title' => $acquirer->title,
  )));
}

/**
 * Global configuration form.
 */
function idealui_form_global_configuration(array &$form_state) {
  $form['ideal_test'] = array(
    '#type' => 'checkbox',
    '#title' => t('Use testing servers only'),
    '#default_value' => variable_get('ideal_test', TRUE),
  );
  $form['ideal_debug'] = array(
    '#type' => 'checkbox',
    '#title' => t('Display log errors on screen'),
    '#default_value' => variable_get('ideal_debug', TRUE),
  );
  $form['ideal_default_expiration_period'] = array(
    '#type' => 'select',
    '#title' => t('Default expiration period'),
    '#options' => array(
      180 => t('3 minutes'),
      240 => t('4 minutes'),
      300 => t('5 minutes'),
      600 => t('10 minutes'),
      900 => t('15 minutes'),
      1200 => t('20 minutes'),
      1800 => t('30 minutes'),
      2400 => t('40 minutes'),
      3000 => t('50 minutes'),
      3600 => t('1 hour'),
    ),
    '#default_value' => variable_get('ideal_default_expiration_period', 900),
    '#required' => TRUE,
  );

  return system_settings_form($form);
}